
--- A build back-end to provide a portable way of building C-based Lua modules.
module("luarocks.build_module", package.seeall)

local fs = require("luarocks.fs")
local path = require("luarocks.path")
local util = require("luarocks.util")
local config = require("luarocks.config")

--- Check if platform was detected
-- @param query string: The platform name to check.
-- @return boolean: true if LuaRocks is currently running on queried platform.
local function is_platform(query)
   assert(type(query) == "string")

   for _, platform in ipairs(config.platforms) do
      if platform == query then
         return true
      end
   end
end

local function execute(...)
   print(table.concat({...}, " "))
   return fs.execute(...)
end

--- Driver function for the "module" build back-end.
-- @param rockspec table: the loaded rockspec.
-- @return boolean or (nil, string): true if no errors ocurred,
-- nil and an error message otherwise.
function run(rockspec)
   assert(type(rockspec) == "table")
   local compile_object, compile_library

   local build = rockspec.build
   local variables = rockspec.variables

   if is_platform("win32") then
      compile_object = function(object, source)
         return execute(variables.CC, "-c", "-Fo"..object, "-I"..variables.LUA_INCDIR, source)
      end
      compile_library = function(library, objects)
         local basename = fs.base_name(library):gsub(".[^.]*$", "")
         local deffile = basename .. ".def"
         local def = io.open(fs.make_path(fs.current_dir(), deffile), "w+")
         def:write("EXPORTS\n")
         def:write("luaopen_"..basename.."\n")
         def:close()
         return execute(variables.LD, "-dll", "-def:"..deffile, "-out:"..library, fs.make_path(variables.LUA_LIBDIR, "lua5.1.lib"), unpack(objects))
      end
   else
      compile_object = function(object, source)
         return execute(variables.CC, "-I"..variables.LUA_INCDIR, "-c", source)
      end
      compile_library = function (library, objects)
         return execute(variables.LD.." "..variables.LIBFLAG, "-o", library, unpack(objects))
      end
   end

   local ok = true
   local built_modules = {}
   local luadir = path.lua_dir(rockspec.name, rockspec.version)
   local libdir = path.lib_dir(rockspec.name, rockspec.version)
   for name, info in pairs(build.modules) do
      local moddir = path.module_to_path(name)
      if type(info) == "string" then
         local ext = info:match(".([^.]+)$")
         if ext == "lua" then
            local dest = fs.make_path(luadir, moddir)
            built_modules[info] = dest
         else
            info = {info}
         end
      end
      if type(info) == "table" then
         local objects = {}
         local sources = info.sources
         if info[1] then sources = info end
         if type(sources) == "string" then sources = {sources} end
         for _, source in ipairs(sources) do
            local object = source:gsub(".[^.]*$", "."..config.obj_extension)
            if not object then
               object = source.."."..config.obj_extension
            end
            ok = compile_object(object, source)
            if not ok then break end
            table.insert(objects, object)
         end
         if not ok then break end
         local module_name = fs.make_path(moddir, fs.base_name(name).."."..config.lib_extension)
         local dest = fs.make_path(libdir, moddir)
         built_modules[module_name] = dest
         ok = compile_library(module_name, objects)
         if not ok then break end
      end
   end
   for name, dest in pairs(built_modules) do
      fs.make_dir(dest)
      ok = fs.copy(name, dest)
      if not ok then break end
   end
   if ok then
      return true
   else
      return nil, "Build error"
   end
end
